from django.core.management.base import BaseCommand
from django.conf import settings

from populate.populator.common.file_manager import FileManager
from populate.populator.common.populator_actions import PopulatorActions


class Command(BaseCommand):
    help = (
        'Populate service-exo-opportunities'
    )

    def handle(self, *args, **options):
        self.stdout.write('\n Populating {}: \n\n'.format(settings.SERVICE_NAME))

        fm = FileManager()
        fm.prepare_regression_files()
        actions = PopulatorActions()
        actions.populate(settings.POPULATE_ALLOWED_POPULATE_ENTITIES)
        fm.clear_files()
        actions.finish_flag()

        self.stdout.write('\n Populated {}: \n\n'.format(settings.SERVICE_NAME))



# -*- coding: utf-8 -*-
"""
License boilerplate should be used here.
"""
# python 3 imports
from __future__ import absolute_import, unicode_literals

# python imports
import logging

# 3rd. libraries imports
from appconf import AppConf

# django imports
from django.conf import settings  # NOQA

logger = logging.getLogger(__name__)


class PopulateConfig(AppConf):
    APP_NAME = 'populate'

    ALLOWED_DATA_FOLDERS = []

    ALLOWED_POPULATE_ENTITIES = []

    REQUIRED_SEQUENCE_RESET_MODELS = []



from abc import ABC

from django.core.management.base import BaseCommand

import yaml
from populator import BASE_DIR


class Manager(ABC):
    cmd = BaseCommand()
    builder = None
    model = None
    attribute = None
    files_path = None

    def get_object(self, value):
        attr_value = value
        if isinstance(value, dict):
            attr_value = value.get(self.attribute)
        try:
            return self.model.objects.get(
                **self.build_expression(attr_value))
        except self.model.DoesNotExist:
            data = self.load_file(attr_value)
            if isinstance(data, dict):
                print('Parsing {0} {1}...'.format(
                    self.model.__name__, attr_value))
                try:
                    object = self.model.objects.get(
                        **self.build_expression(attr_value))
                    self.output_warn(object, 'already exists!')
                    return object
                except self.model.DoesNotExist:
                    if isinstance(value, dict):
                        data.update(value)
                    object = self.builder(data).create_object()
                    self.output_success(object, 'created!')
                    return object
            else:
                return data

    def build_expression(self, attr_value):
        if self.attribute == 'uuid':
            return {'{}'.format(self.attribute): attr_value.replace('_', '-')}
        return {'{}__iexact'.format(self.attribute): attr_value}

    def load_file(self, value):
        path = '{base_dir}{file_path}'.format(
            base_dir=BASE_DIR, file_path=self.files_path)
        file_obj = open('{}{}.yml'.format(path, self.normalize(value)))
        data = yaml.load(file_obj)
        file_obj.close()
        return data

    def normalize(self, value):
        return value.lower().replace('-', '_').replace(' ', '_')

    def delete_all(self):
        self.model.objects.all().delete()

    def output_success(self, object, msg):
        self.cmd.stdout.write(
            self.cmd.style.SUCCESS(
                self.model.__name__ + ' ' + getattr(
                    object, self.attribute).__str__() + ' ' + msg))

    def output_warn(self, object, msg):
        self.cmd.stdout.write(
            self.cmd.style.WARNING(
                self.model.__name__ + ' ' + getattr(
                    object, self.attribute) + ' ' + msg))



import os

from django.core.management.base import BaseCommand
from django.core.management import call_command
from django.db import connection
from django.conf import settings

from populate.populator.common.bulk_operations import BulkOperations
from populate.populator.status import PopulateStatus


class PopulatorActions:
    def __init__(self):
        self.status = PopulateStatus()
        self.cmd = BaseCommand()

    def get_status(self):
        return self.status.get_status()

    def populate(self, items):
        results = {}
        for item in items:
            results[item] = BulkOperations(entity=item).populate()
        self.sql_sequence_reset()
        self.status.set_populated()
        return results

    def init(self):
        self.status.set_initialized()
        # Execute migrations
        call_command('migrate')

    def finish_flag(self):
        print('\n\n\n\n\n\n')
        self.cmd.stdout.write(
            self.cmd.style.SUCCESS('Populator succesfully finished!'))

    def sql_sequence_reset(self):
        if settings.POPULATE_REQUIRED_SEQUENCE_RESET_MODELS:
            sql = call_command(
                'sqlsequencereset',
                *settings.POPULATE_REQUIRED_SEQUENCE_RESET_MODELS,
                no_color=True)
            with connection.cursor() as cursor:
                cursor.execute(sql)

    def is_sql(self, file):
        _, extension = os.path.splitext(file)
        return extension == '.sql'



from datetime import timedelta

from django.utils import timezone

import yaml
from dateutil import parser

from .common.errors import ConfigurationError
from populator import loader as app_loader # noqa


def datetime_constructor(loader, node):
    item = loader.construct_scalar(node)
    if not isinstance(item, str) or not item:
        raise ConfigurationError(
            'value {} cannot be interpreted as date range'.format(item))
    return parser.parse(item)


def timedelta_constructor(loader, node):
    item = loader.construct_scalar(node)

    if not isinstance(item, str) or not item:
        raise ConfigurationError(
            'value {} cannot be interpreted as date range'.format(item))
    num, typ = item[:-1], item[-1].lower()
    if not num.lstrip('-').isdigit():
        raise ConfigurationError(
            'value {} cannot be interpreted as date range'.format(item))

    num = int(num)
    tmdt = None
    if typ == 'd':
        tmdt = timedelta(days=num)
    elif typ == 'h':
        tmdt = timedelta(seconds=num * 3600)
    elif typ == 'w':
        tmdt = timedelta(days=num * 7)
    elif typ == 'm':
        tmdt = timedelta(seconds=num * 60)
    elif typ == 's':
        tmdt = timedelta(seconds=num)
    return timezone.now() + tmdt


# Global constructors
yaml.Loader.add_constructor('!datetime', datetime_constructor)
yaml.Loader.add_constructor('!timedelta', timedelta_constructor)



class PopulateStatus:
    status = False

    def set_initialized(self):
        self.status = False

    def set_populated(self):
        self.status = True

    def get_status(self):
        return self.status



from django.conf import settings


def find_tuple_values(tpl, values):
    """
    Allow to find a key which is related to a value in python tuple
    """
    return [str(obj[0]) for obj in tpl if obj[1] in values]


def get_db_credentials():
    credentials = settings.DATABASES.get('default', None)
    result = None

    if credentials:
        result = {
            'HOST': credentials.get('HOST', ''),
            'USER': credentials.get('USER', ''),
            'PASS': credentials.get('PASSWORD', ''),
            'NAME': credentials.get('NAME', ''),
            'PORT': credentials.get('PORT', ''),
        }

    return result



import importlib
import os
import glob
import stringcase
from uuid import UUID

from django.conf import settings


class BulkOperations:
    def __init__(self, entity):
        self.entity = entity
        self.model = '{0}Manager'.format(
            stringcase.titlecase(entity).replace(" ", ""))
        self.root_path = settings.BASE_DIR

    def populate(self):
        allowed = settings.POPULATE_ALLOWED_POPULATE_ENTITIES
        if self.entity in allowed:
            result = []
            for item in self.get_item_list():
                module = importlib.import_module(
                    'populator.{0}.{0}_manager'.format(self.entity))
                class_ = getattr(module, self.model)
                result.append(class_().get_object(item).id)
        else:
            result = 'Skipped'
        return result

    def is_uuid(self, uuid_string):
        try:
            UUID(uuid_string, version=4)
        except ValueError:
            return False
        return True

    def get_item_list(self):
        list = []
        folder = '{0}/populator/{1}/files/'.format(self.root_path, self.entity)
        files = glob.glob(folder + '**/*.yml', recursive=True)
        for file in files:
            filename, _ = os.path.splitext(os.path.basename(file))
            if self.is_uuid(filename.replace('_', '-')):
                list.append(filename)
            else:
                list.append(stringcase.titlecase(filename))
        return list



import os
import shutil

from django.core.management.base import BaseCommand
from django.conf import settings


class FileManager:

    def __init__(self):
        self.target_path = '{}/populator'.format(settings.BASE_DIR)
        self.regression_path = '{0}/populator/data/'.format(settings.BASE_DIR)
        self.cmd = BaseCommand()

    def clear_files(self):
        for f in settings.POPULATE_ALLOWED_DATA_FOLDERS:
            f = '{0}/{1}/files'.format(self.target_path, f)
            self.delete(f)

    def delete(self, src):
        for file in os.listdir(src):
            src_file = os.path.join(src, file)
            if self.is_yml(src_file):
                self.cmd.stdout.write(
                    self.cmd.style.ERROR('Removed file: {0}'.format(src_file)))
                os.unlink(src_file)
            elif self.is_dir(src_file):
                self.delete(src_file)
                os.rmdir(src_file)

    def prepare_regression_files(self):
        for folder in os.listdir(self.regression_path):
            src_path = '{0}/{1}'.format(self.regression_path, folder)
            dst_path = '{0}/{1}/files'.format(self.target_path, folder)
            self.copytree(src_path, dst_path)

    def copytree(self, src, dst):
        for file in os.listdir(src):
            src_file = os.path.join(src, file)
            dst_file = os.path.join(dst, file)
            if self.is_yml(src_file):
                shutil.copy2(src_file, dst_file)
                self.cmd.stdout.write(
                    self.cmd.style.SUCCESS('{0}'.format(dst_file)))
            elif self.is_dir(src_file):
                os.makedirs(dst_file)
                self.copytree(src_file, dst_file)

    def is_yml(self, file):
        _, extension = os.path.splitext(file)
        return extension == '.yml'

    def is_dir(self, file):
        return os.path.isdir(file)
